
namespace Zipr_SDK
{
typedef std::map<libIRDB::Instruction_t*,Zipr_SDK::RangeAddress_t> InstructionLocationMap_t;

enum ZiprPreference {
	None,
	Must,	
};
class Zipr_t;

class ZiprPluginInterface_t
{
	public:
		virtual void PinningBegin()
		{
		}
		virtual void PinningEnd()
		{
		}
		virtual void DollopBegin()
		{
		}
		virtual void DollopEnd()
		{
		}
		virtual void CallbackLinkingBegin()
		{
		}
		virtual void CallbackLinkingEnd()
		{
		}

		virtual ZiprPreference AddressDollop(const Dollop_t *dollop, const RangeAddress_t &source, Range_t &place, bool &coalesce)
		{
			return None;
		}

		virtual bool WillPluginPlop(libIRDB::Instruction_t*)
		{
			return false;
		}
		virtual size_t WorstCaseInsnSize(libIRDB::Instruction_t*, bool, Zipr_t *)
		{
			return 0;
		}
		/*
		 * InsnPaddingSize()
		 *
		 * A plugin that wants to put padding into a
		 * dollop entry before/after an instruction 
		 * should return the size of that padding in this
		 * function.
		 */
		virtual size_t InsnPaddingSize(libIRDB::Instruction_t*, Zipr_t *)
		{
			return 0;
		}

		/*
		 * PlopDollopEntry()
		 *
		 * This function is invoked by Zipr for a plugin to 
		 * plop a dollop entry. The entry to plop is given as
		 * the first parameter. The plugin writer can use
		 * it's Place() function to determine the beginning
		 * of the space allocated for this dollop entry. 
		 *
		 * Because multiple plugins may want to plop the same
		 * dollop entry, the current address of the instruction
		 * within the entry is passed as the second parameter.
		 * If the plugin moves that instruction, it must update
		 * the parameter. In order to know the length of the
		 * instruction, there is the third parameter.
		 *
		 * Finally, to access the other functions that Zipr
		 * provides, the fourth parameter is available.
		 *
		 * As output, the implementation must provide the 
		 * address of the memory that "concludes" the 
		 * dollop entry.
		 */
		virtual RangeAddress_t PlopDollopEntry(Zipr_SDK::DollopEntry_t *,
			RangeAddress_t &,
			size_t,
			Zipr_t *)
		{
			return 0;
		}
		virtual std::string ToString()
		{
			return "NamelessPlugin";
		}
		virtual ZiprOptionsNamespace_t *RegisterOptions(ZiprOptionsNamespace_t *)
		{
			return new ZiprOptionsNamespace_t("");
		}

		virtual ZiprPreference RetargetPin(const RangeAddress_t &, const Zipr_SDK::Dollop_t *, RangeAddress_t &)
		{
			return None;
		}
		virtual ZiprPreference RetargetCallback(const RangeAddress_t &, const Zipr_SDK::DollopEntry_t *, RangeAddress_t &)
		{
			return None;
		}
		virtual RangeAddress_t PlaceScoopsBegin(const RangeAddress_t max_addr)
		{
			return max_addr;
		}
		virtual RangeAddress_t PlaceScoopsEnd(const RangeAddress_t max_addr)
		{
			return max_addr;
		}

};

}
